<?php
/*
Plugin Name: Migrate Emails via IMAP
Plugin URI: http://www.codingconcepts.net
Description: A custom plug-in created by Brendan Carr
Author: Brendan Carr
Version: 1.0
Author URI: http://www.codingconcepts.net

Copyright 2015 Coding Concepts

*/


register_activation_hook(__FILE__, 'cc_migrate_emails_install');
register_deactivation_hook(__FILE__, 'cc_migrate_emails_uninstall');
add_action('admin_menu', 'cc_migrate_emails_settings_page');

function cc_migrate_emails_install() {
  
}

function cc_migrate_emails_uninstall() {
  
}

function cc_migrate_emails_settings_page() {
  add_management_page('Migrate Emails', 'Migrate Emails', 'manage_options', __FILE__, 'cc_migrate_emails_admin_page');
}

function cc_migrate_emails_admin_page() {
?>
    <div class="wrap">
      <h1>Migrate Emails via IMAP Plugin</h1>
      <p>This simple plugin will migrate emails between two IMAP mailboxes. Simply click the <em>"Migrate Emails"</em> below after filling in details. Recommend incrementing the starting and ending email numbers, try to do 1000 at a time at most, depending on how much memory you have available on your server. It may still get stuck on large attachments!</p>
      <h2>License</h2>
      <p>By using this plugin, you acknowledge that this plugin (and all its source code) is distributed <strong style="color:#990000">AS IS</strong> with no implicit or explicit warranty as to the plugin's fitness for any specific purpose, 
        and is released under the <a href="http://www.gnu.org/licenses/gpl-2.0.txt">GPLv2</a> license.</p>
      <p>If you agree to the licensing terms above, click the Migrate emails button now to begin.
      <form action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>" method="post">
	    From Hostname: <input type="text" name="fhostname" placeholder="hostname" value="<?php echo $_POST['fhostname']; ?>"><br />
	    From Username: <input type="text" name="fusername" placeholder="username" value="<?php echo $_POST['fusername']; ?>"><br />
	    From Password: <input type="text" name="fpassword" placeholder="password" value="<?php echo $_POST['fpassword']; ?>"><br />
	    Destination Hostname: <input type="text" name="dhostname" placeholder="hostname" value="<?php echo $_POST['dhostname']; ?>"><br />
        Destination Username: <input type="text" name="dusername" placeholder="username" value="<?php echo $_POST['dusername']; ?>"><br />
        Destination Password: <input type="text" name="dpassword" placeholder="password" value="<?php echo $_POST['dpassword']; ?>"><br />
		Starting Email Number: <input type="text" name="snumber" value="1" placeholder="start" value="<?php echo $_POST['snumber']; ?>"><br />
		Ending Email Number: <input type="text" name="enumber" value="10" placeholder="end" value="<?php echo $_POST['enumber']; ?>"><br />
		<input name="migrate-clicked" type="hidden" value="1" />
        <input type="submit" value="Migrate Emails" />
      </form>
    </p>
    </div>
<?php
	if (isset($_POST['migrate-clicked'])) {
	
		$fhostname = $_POST['fhostname'];
		$fusername = $_POST['fusername'];
		$fpassword = $_POST['fpassword'];
		$dhostname = $_POST['dhostname'];
		$dusername = $_POST['dusername'];
		$dpassword = $_POST['dpassword'];
		$snumber = $_POST['snumber'];
		$enumber = $_POST['enumber'];
		
		
		$fromMboxServerPath = "{".$fhostname."/notls/imap:143}";
		$fromMboxMailboxPath = "INBOX";
		
		$toMboxServerPath = "{".$dhostname."/notls/imap:143}";
		$toMboxMailboxPath = "INBOX";
		
		$fromMboxMailAddress = $fusername;
		$fromMboxMailPass = $fpassword;
		
		$toMboxMailAddress = $dusername;
		$toMboxMailPass = $dpassword;
		
		$fromMboxConnStr = $fromMboxServerPath.$fromMboxMailboxPath;
		$toMboxConnStr = $toMboxServerPath.$toMboxMailboxPath;
		
		//Increment these (ie: 1001 to 2000) to continue transferring
		$fetchStartSeq = $snumber;
		$fetchEndSeq = $enumber;
		
		function myLog($str)
		{
		    echo "Log [".date('Y-m-d H:i:s')."]: $str\n<BR>";
		}
		
		myLog("Connecting to mailbox");
		
		function mboxConn($connstr,$addr,$pass)
		{
		    if(!($mbox = @imap_open($connstr, $addr, $pass)))
		    {
		        myLog("Error: ".imap_last_error());
		        die;
		    }
		    else
		    {
		        myLog("Connected to: $addr $connstr");
		        return $mbox;
		    }
		}
		
		function mboxCheck($mbox)
		{
		    if(!($mbox_data = imap_check($mbox)))
		    {
		       myLog("Error: ".imap_last_error());
		       die;   
		    }
		    else
		    {
		        myLog("Mailbox check ".$mbox_data->Mailbox." - OK");
		        myLog($mbox_data->Nmsgs." messages present");
		        return $mbox_data->Nmsgs;
		    }
		}
		
		$fromMbox = mboxConn($fromMboxConnStr, $fromMboxMailAddress, $fromMboxMailPass);
		$toMbox = mboxConn($toMboxConnStr, $toMboxMailAddress, $toMboxMailPass);
		
		$fromMboxCount = mboxCheck($fromMbox);
		$toMboxCount = mboxCheck($toMbox);
		
		/**
		* Loop on mails
		*/
		
		$fetchStartUID = imap_uid($fromMbox,$fetchStartSeq);
		if ($fromMboxCount < $fetchEndSeq)
		{
		    $fetchEndSeq = $fromMboxCount;
		}
		$fetchEndUID = imap_uid($fromMbox,$fetchEndSeq);
		
		/**
		* Loop on mails
		*/
		
		myLog("Performing task of migration from UID [$fetchStartUID] to UID [$fetchEndUID]");

		for ($i=$fetchStartSeq;$i<=$fetchEndSeq;$i++)
		{
		    $pfx = "Msg #$i : ";
		    $h = imap_header($fromMbox, $i);
		    $fh = imap_fetchheader($fromMbox, $i);
			$headerinfo = imap_headerinfo($fromMbox, $i);
			$internal_date=date('d-M-Y H:i:s O',$headerinfo->udate);
		    //MyLog($fh."<BR><BR>");
		    $fb = imap_body($fromMbox, $i);
		    $message = $fh.$fb;
		
		    $msgUID = imap_uid($fromMbox,$i);
		
		    $struct = imap_fetchstructure ($fromMbox, $i);
		
		    /**
		     * We do some logging
		     */
			 
			 //echo "<textarea>";
			 //print_r($headerinfo);
			 //echo "</textarea>";
			 
			//myLog($pfx."fetchheader: [".serialize($fh)."]"); 
			myLog($pfx."Date: [".$internal_date."]"); 
			//myLog($pfx."headers: [".serialize($headers)."]"); 
		    myLog($pfx."UID [".$msgUID."] SEQ [".imap_msgno($fromMbox,$msgUID)."] Flags: [". $h->Unseen . $h->Recent . $h->Deleted . $h->Answered . $h->Draft . $h->Flagged."]");
		    myLog($pfx."From: [". htmlspecialchars($h->fromaddress) . "] To: [".htmlspecialchars($h->toaddress)."]");
		    myLog($pfx."Subject: [$h->subject]");
		
		    /**
		     * Here you do whaterver you need with your email
		     */
		
		    /**
		     * Backup email
		     */
		    if (!($ret = imap_append($toMbox,$toMboxServerPath.$toMboxMailboxPath,$message,"\SEEN",$internal_date))) 
		    {
		        myLog("Error: ".imap_last_error());
		        die;
		    }
		    else
		    {
		        myLog("everything ok, mail [$fetchStartUID:$fetchEndUID] downloaded and moved in $newMailboxNameMOVE");
		    }
		}
		
		/**
		* End
		*/
		
		imap_close($fromMbox);
		imap_close($toMbox);
		
		myLog("Connection closed");
	}
}

?>